//+---------------------------------------------------------------------+
//|                                                   StochasticRSI.mq5 |
//|                                                                     |
//| StochasticRSI                                                       |
//|                                                                     |
//| Algorithm taken from book                                           |
//|     "Cybernetics Analysis for Stock and Futures"                    |
//| by John F. Ehlers                                                   |
//|                                                                     |
//|                                                 contact@mqlsoft.com |
//|                                             http://www.mqlsoft.com/ |
//+---------------------------------------------------------------------+
//|          SmoothAlgorithms.mqh |
//|   (): __\MQL5\Include         |
//+---------------------------------------------------------------------+
//----  
#property copyright "Coded by Witold Wozniak"
//----  
#property link      "www.mqlsoft.com"
//----   
#property version   "1.00"
//----     
#property indicator_separate_window 
//----        
#property indicator_buffers 2
//----    
#property indicator_plots   2
//+----------------------------------------------+
//|     StoRSI       |
//+----------------------------------------------+
//----   1   
#property indicator_type1   DRAW_LINE
//----        Red
#property indicator_color1  Red
//----   1 -  
#property indicator_style1  STYLE_SOLID
//----    1  1
#property indicator_width1  1
//----    
#property indicator_label1  "Stochastic RSI"
//+----------------------------------------------+
//|     Trigger      |
//+----------------------------------------------+
//----   2   
#property indicator_type2   DRAW_LINE
//----         Blue
#property indicator_color2  Blue
//----   2 -  
#property indicator_style2  STYLE_SOLID
//----    2  1
#property indicator_width2  1
//----     
#property indicator_label2  "Trigger"
//+----------------------------------------------+
//|     |
//+----------------------------------------------+
#property indicator_level1 0.7
#property indicator_level2 0.5
#property indicator_level3 0.3
#property indicator_levelcolor Gray
#property indicator_levelstyle STYLE_DASHDOTDOT
//+----------------------------------------------+
//|       |
//+----------------------------------------------+
#property indicator_minimum 0
#property indicator_maximum 1
//+----------------------------------------------+
//|                    |
//+----------------------------------------------+
input int RSILength=8;
input int StocLength= 8;
input int WMALength = 8;
input int Shift=0;      //       
//+----------------------------------------------+
//----   ,    
//----      
double StoRSIBuffer[];
double TriggerBuffer[];
//----      
int RSI_Handle;
//----      
int rsi_rates_total,sto_rates_total,min_rates_total;
//+------------------------------------------------------------------+
//   CMoving_Average                                   | 
//+------------------------------------------------------------------+  
#include <SmoothAlgorithms.mqh> 
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+  
void OnInit()
  {
//----     
   rsi_rates_total=RSILength;
   sto_rates_total=rsi_rates_total+StocLength;
   min_rates_total=sto_rates_total+WMALength+1;

//----    RSI
   RSI_Handle=iRSI(NULL,0,RSILength,PRICE_CLOSE);
   if(RSI_Handle==INVALID_HANDLE) Print("      RSI");

//----    StoRSIBuffer[]   
   SetIndexBuffer(0,StoRSIBuffer,INDICATOR_DATA);
//----    1    Shift
   PlotIndexSetInteger(0,PLOT_SHIFT,Shift);
//----       1  min_rates_total
   PlotIndexSetInteger(0,PLOT_DRAW_BEGIN,min_rates_total);
//----    ,      
   ArraySetAsSeries(StoRSIBuffer,true);

//----    TriggerBuffer[]   
   SetIndexBuffer(1,TriggerBuffer,INDICATOR_DATA);
//----    2    Shift
   PlotIndexSetInteger(1,PLOT_SHIFT,Shift);
//----       2  min_rates_total+1
   PlotIndexSetInteger(1,PLOT_DRAW_BEGIN,min_rates_total+1);
//----    ,      
   ArraySetAsSeries(TriggerBuffer,true);

//----      
   string shortname;
   StringConcatenate(shortname,"Stochastic RSI(",RSILength,", ",StocLength,", ",WMALength,", ",Shift,")");
//---           
   IndicatorSetString(INDICATOR_SHORTNAME,shortname);
//---     
   IndicatorSetInteger(INDICATOR_DIGITS,2);
//----
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,    //       
                const int prev_calculated,//       
                const datetime &time[],
                const double &open[],
                const double& high[],     //       
                const double& low[],      //        
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
  {
//----       
   if(BarsCalculated(RSI_Handle)<rates_total || rates_total<min_rates_total) return(0);

//----    
   int limit,to_copy,maxbar,bar;
   double tmp,hh,ll,Value1,Value2,Value3,RSI[];

//----    ,     
   ArraySetAsSeries(RSI,true);

//----    first    
   if(prev_calculated>rates_total || prev_calculated<=0) //      
      limit=rates_total-1-rsi_rates_total; //      
   else limit=rates_total-prev_calculated; //      

   to_copy=limit+min_rates_total+1;
   maxbar=rates_total-1-sto_rates_total;

//----    CMoving_Average   MASeries_Cls.mqh
   static CMoving_Average MA;

//----      
   if(CopyBuffer(RSI_Handle,0,0,to_copy,RSI)<=0) return(0);

//----    
   for(bar=limit; bar>=0 && !IsStopped(); bar--)
     {
      hh = RSI[bar];
      ll = RSI[bar];

      for(int iii=0; iii<StocLength; iii++)
        {
         tmp= RSI[bar+iii];
         hh = MathMax(hh, tmp);
         ll = MathMin(ll, tmp);
        }

      Value1 = RSI[bar] - ll;
      Value2 = hh - ll;

      if(Value2 != 0.0) Value3= Value1/Value2;
      else Value3= 0.0;

      StoRSIBuffer[bar]=MA.MASeries(maxbar,prev_calculated,rates_total,WMALength,MODE_LWMA,Value3,bar,true);
      TriggerBuffer[bar]=StoRSIBuffer[bar+1];
     }
//----     
   return(rates_total);
  }
//+------------------------------------------------------------------+
